@@ -1,9 +1,11 @@
 #' Internal function to generate a progress bar for the bootstrap option
 #' @param int integer
+#' @param tot tot
 #' @returns No returns, just prints dots for each bootstrap replication
 #' @noRd
 progressBar <- function(
-    int
+    int,
+    tot
     ) {
     cat(".")
     if (int %% 5 == 0) {
@@ -12,4 +14,7 @@ progressBar <- function(
     if (int %% 70 == 0) {
         cat("\n")
     }
+    if (int == tot) {
+        cat("\n")
+    }
 }
\ No newline at end of file
